Package: rEDM
Type: Package
Title: Empirical Dynamic Modeling ('EDM')
Version: 1.15.4
Date: 2024-04-05
Authors@R: c( person("Joseph", "Park", role = c("aut", "cre"),
                     email = "JosephPark@IEEE.org",
                     comment = c(ORCID = "0000-0001-5411-1409")),
              person("Cameron", "Smith", role = c("aut"),
                     email = "cos008@ucsd.edu",
                     comment = c(ORCID = "0000-0003-0020-5607")),
              person("George", "Sugihara", role = c("aut", "ccp"), 
                     comment = c(ORCID = "0000-0002-2863-6946")),
              person("Ethan", "Deyle", role = c("aut"),
                     comment = c(ORCID = "0000-0001-8704-8434")),
              person("Erik", "Saberski", role = c("ctb"),
                     comment = c(ORCID = "0000-0002-6475-6187")),
              person("Hao", "Ye", role = c("ctb"), 
                     comment = c(ORCID = "0000-0002-8630-1458")),
              person("The Regents of the University of California",
                     role = c("cph") ) )
Maintainer: Joseph Park <JosephPark@IEEE.org>
Description: An implementation of 'EDM' algorithms based on research software developed for internal use at the Sugihara Lab ('UCSD/SIO').  The package is implemented with 'Rcpp' wrappers around the 'cppEDM' library.  It implements the 'simplex' projection method from Sugihara & May (1990) <doi:10.1038/344734a0>, the 'S-map' algorithm from Sugihara (1994) <doi:10.1098/rsta.1994.0106>, convergent cross mapping described in Sugihara et al. (2012) <doi:10.1126/science.1227079>, and, 'multiview embedding' described in Ye & Sugihara (2016) <doi:10.1126/science.aag0863>.
License: BSD_2_clause + file LICENSE
LazyData: true
LazyLoad: yes
Imports: methods, Rcpp (>= 1.0.1)
LinkingTo: Rcpp, RcppThread
Suggests: knitr, rmarkdown, formatR
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2024-04-05 17:24:24 UTC; jpark
Author: Joseph Park [aut, cre] (<https://orcid.org/0000-0001-5411-1409>),
  Cameron Smith [aut] (<https://orcid.org/0000-0003-0020-5607>),
  George Sugihara [aut, ccp] (<https://orcid.org/0000-0002-2863-6946>),
  Ethan Deyle [aut] (<https://orcid.org/0000-0001-8704-8434>),
  Erik Saberski [ctb] (<https://orcid.org/0000-0002-6475-6187>),
  Hao Ye [ctb] (<https://orcid.org/0000-0002-8630-1458>),
  The Regents of the University of California [cph]
Repository: CRAN
Date/Publication: 2024-04-06 10:30:03 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 18:44:27 UTC; unix
Archs: rEDM.so.dSYM
