useDynLib( rEDM, .registration = TRUE )

export( Simplex   )
export( SMap      )
export( CCM       )
export( Multiview )
export( Embed     )
export( MakeBlock )
export( ComputeError     )
export( EmbedDimension   )
export( PredictInterval  )
export( PredictNonlinear )
export( SurrogateData    )

# Legacy functions
# export( ccm           )
# export( block_lnlp    )
# export( s_map         )
# export( simplex       )
# export( multiview     )
# export( make_block    )
# export( compute_stats )
# export( make_surrogate_data )

importFrom("grDevices", "dev.cur", "dev.new", "dev.list")
importFrom("graphics", "abline", "legend", "lines", "mtext", "par", "plot")
importFrom("utils", "data", "read.csv")
importFrom("stats", "fft", "predict", "rnorm", "runif", "sd", "smooth.spline", "cov", "pnorm")
import( methods )
import( Rcpp )
